import { useState } from "react";
import Button from "./Button";
import ChangeRole, { ProjectRoleChange } from "./ChangeRole";

export default function ChangeRoleView(props: {
  projectName: string;
  username: string;
  currentRole: string;
}): JSX.Element {
  const [selectedRole, setSelectedRole] = useState<
    "project_manager" | "member" | ""
  >("");

  const handleRoleChange = (
    event: React.ChangeEvent<HTMLInputElement>,
  ): void => {
    if (event.target.value === "member") {
      setSelectedRole(event.target.value);
    } else if (event.target.value === "project_manager") {
      setSelectedRole(event.target.value);
    }
  };

  const handleSubmit = (event: React.FormEvent<HTMLFormElement>): void => {
    console.log("Cur: " + props.currentRole + " " + "new: " + selectedRole);
    event.preventDefault();
    if (selectedRole === props.currentRole) {
      alert(`Already ${props.currentRole}, nothing changed`);
      return;
    }
    const roleChangeInfo: ProjectRoleChange = {
      username: props.username,
      projectname: props.projectName,
      role: selectedRole,
    };
    ChangeRole(roleChangeInfo);
  };

  return (
    <div className="overflow-auto">
      <h1 className="font-bold text-[20px]">Select role:</h1>
      <form onSubmit={handleSubmit}>
        <div className="py-1 px-1 w-full self-start text-left font-medium overflow-auto border-2 border-black rounded-2xl">
          <label className="hover:cursor-pointer hover:font-bold">
            <input
              type="radio"
              value="project_manager"
              checked={selectedRole === "project_manager"}
              onChange={handleRoleChange}
              className="m-2"
            />
            Project manager
          </label>
          <br />
          <label className="hover:cursor-pointer hover:font-bold">
            <input
              type="radio"
              value="member"
              checked={selectedRole === "member"}
              onChange={handleRoleChange}
              className="m-2 hover:cursor-pointer"
            />
            Member
          </label>
        </div>
        <Button
          text="Change"
          onClick={(): void => {
            return;
          }}
          type="submit"
        />
      </form>
    </div>
  );
}
