import { useState, useEffect } from "react";
import { WeeklyReport } from "../Types/goTypes";
import { api } from "../API/API";
import { useParams, useNavigate } from "react-router-dom";
import Button from "./Button";

/**
 * Renders the component for editing a weekly report.
 * @returns JSX.Element
 */

//This component does not yet work as intended. It is supposed to display the weekly report of a user in a project.
export default function OtherUsersTR(): JSX.Element {
  const [week, setWeek] = useState(0);
  const [developmentTime, setDevelopmentTime] = useState(0);
  const [meetingTime, setMeetingTime] = useState(0);
  const [adminTime, setAdminTime] = useState(0);
  const [ownWorkTime, setOwnWorkTime] = useState(0);
  const [studyTime, setStudyTime] = useState(0);
  const [testingTime, setTestingTime] = useState(0);
  const [reportId, setReportId] = useState(0);

  const token = localStorage.getItem("accessToken") ?? "";
  const { projectName } = useParams();
  const { username } = useParams();
  const { fetchedWeek } = useParams();
  const { signedOrUnsigned } = useParams();
  console.log(projectName, username, fetchedWeek, signedOrUnsigned);

  useEffect(() => {
    const fetchUsersWeeklyReport = async (): Promise<void> => {
      const response = await api.getWeeklyReport(
        projectName ?? "",
        fetchedWeek?.toString() ?? "0",
        token,
        username ?? "",
      );

      if (response.success) {
        const report: WeeklyReport = response.data ?? {
          reportId: 0,
          userId: 0,
          projectId: 0,
          week: 0,
          developmentTime: 0,
          meetingTime: 0,
          adminTime: 0,
          ownWorkTime: 0,
          studyTime: 0,
          testingTime: 0,
        };
        setReportId(report.reportId);
        setWeek(report.week);
        setDevelopmentTime(report.developmentTime);
        setMeetingTime(report.meetingTime);
        setAdminTime(report.adminTime);
        setOwnWorkTime(report.ownWorkTime);
        setStudyTime(report.studyTime);
        setTestingTime(report.testingTime);
      } else {
        console.error("Failed to fetch weekly report:", response.message);
      }
    };

    void fetchUsersWeeklyReport();
  });

  const handleUnsignWeeklyReport = async (): Promise<boolean> => {
    const response = await api.unsignReport(reportId, token);
    console.log(response);
    console.log(reportId);
    if (response.success) {
      return true;
    } else {
      return false;
    }
  };

  const handleDeleteWeeklyReport = async (): Promise<boolean> => {
    const response = await api.deleteWeeklyReport(reportId, token);
    console.log(response);
    if (response.success) {
      return true;
    }
    return false;
  };
  const navigate = useNavigate();

  return (
    <>
      <h1 className="text-[30px] font-bold">{username}&apos;s Report</h1>
      <div className="border-4 border-black bg-white flex flex-col justify-start min-h-[65vh] h-fit w-[50vw] rounded-3xl overflow-scroll space-y-[2vh] p-[30px] items-center">
        <div className="flex flex-col items-center">
          <div className="flex flex-col w-1/2 border-b-2 border-black items-center justify-center">
            <h1 className="font-bold text-[30px]"> Week: {week}</h1>
          </div>

          <table className="w-full text-center divide-y divide-x divide-white text-[30px]">
            <thead>
              <tr>
                <th className="w-1/2 py-2 border-b-2 border-black">Activity</th>
                <th className="w-1/2 py-2 border-b-2 border-black">
                  Total Time (min)
                </th>
              </tr>
            </thead>
            <tbody className="divide-y divide-black">
              <tr className="h-[10vh]">
                <td>Development</td>
                <td>
                  <input
                    type="text"
                    min="0"
                    className="border-2 border-black rounded-md text-center w-1/2"
                    value={developmentTime === 0 ? "" : developmentTime}
                    readOnly
                  />
                </td>
              </tr>
              <tr className="h-[10vh]">
                <td>Meeting</td>
                <td>
                  <input
                    type="text"
                    min="0"
                    className="border-2 border-black rounded-md text-center w-1/2"
                    value={meetingTime === 0 ? "" : meetingTime}
                    readOnly
                  />
                </td>
              </tr>
              <tr className="h-[10vh]">
                <td>Administration</td>
                <td>
                  <input
                    type="text"
                    min="0"
                    className="border-2 border-black rounded-md text-center w-1/2"
                    value={adminTime === 0 ? "" : adminTime}
                    readOnly
                  />
                </td>
              </tr>
              <tr className="h-[10vh]">
                <td>Own Work</td>
                <td>
                  <input
                    type="text"
                    min="0"
                    className="border-2 border-black rounded-md text-center w-1/2"
                    value={ownWorkTime === 0 ? "" : ownWorkTime}
                    readOnly
                  />
                </td>
              </tr>
              <tr className="h-[10vh]">
                <td>Studies</td>
                <td>
                  <input
                    type="text"
                    min="0"
                    className="border-2 border-black rounded-md text-center w-1/2"
                    value={studyTime === 0 ? "" : studyTime}
                    readOnly
                  />
                </td>
              </tr>
              <tr className="h-[10vh]">
                <td>Testing</td>
                <td>
                  <input
                    type="text"
                    min="0"
                    className="border-2 border-black rounded-md text-center w-1/2"
                    value={testingTime === 0 ? "" : testingTime}
                    readOnly
                  />
                </td>
              </tr>
            </tbody>
          </table>
          <div className="flex space-x-4">
            {signedOrUnsigned === "signed" && (
              <Button
                text="Unsign Report"
                onClick={(): void => {
                  void (async (): Promise<void> => {
                    const success = await handleUnsignWeeklyReport();
                    if (success) {
                      alert("Report successfully unsigned!");
                      navigate(-1);
                    } else {
                      alert("Failed to unsign report");
                      return;
                    }
                  })();
                }}
                type={"button"}
              />
            )}
            <Button
              text="Delete Time Report"
              onClick={(): void => {
                void (async (): Promise<void> => {
                  const confirmDelete = window.confirm(
                    "Are you sure you want to delete this report? This action cannot be undone.",
                  );
                  if (!confirmDelete) {
                    return;
                  }
                  const success = await handleDeleteWeeklyReport();
                  if (success) {
                    alert("Report successfully deleted!");
                    navigate(-1);
                  } else {
                    alert("Failed to delete report");
                    return;
                  }
                })();
              }}
              type={"button"}
            />
          </div>
        </div>
      </div>
    </>
  );
}
