import { useState } from "react";
import { NewProject } from "../Types/goTypes";
import ProjectInfoModal from "./ProjectInfoModal";
import MemberInfoModal from "./MemberInfoModal";
import InputField from "./InputField";

/**
 * A list of projects for admin manage projects page, that sets an onClick
 * function for eact project <li> element, which displays a modul with
 * user info.
 * @param props - An array of projects to display
 * @returns {JSX.Element} The project list
 * @example
 * const projects: NewProject[] = [{ name: "Project", description: "New" }];
 * return <ProjectListAdmin projects={projects} />
 */

export function ProjectListAdmin(props: {
  projects: NewProject[];
}): JSX.Element {
  const [projectModalVisible, setProjectModalVisible] = useState(false);
  const [projectName, setProjectName] = useState("");
  const [userModalVisible, setUserModalVisible] = useState(false);
  const [username, setUsername] = useState("");
  const [userRole, setUserRole] = useState("");
  const [search, setSearch] = useState("");

  const handleClickUser = (username: string, userRole: string): void => {
    setUsername(username);
    setUserRole(userRole);
    setUserModalVisible(true);
  };

  const handleClickProject = (projectname: string): void => {
    setProjectName(projectname);
    setProjectModalVisible(true);
  };

  const handleCloseProject = (): void => {
    setProjectName("");
    setProjectModalVisible(false);
  };

  const handleCloseUser = (): void => {
    setUsername("");
    setUserRole("");
    setUserModalVisible(false);
  };

  return (
    <>
      <h1 className="font-bold text-[30px] mb-[20px]">Manage Projects</h1>
      {projectModalVisible && (
        <ProjectInfoModal
          onClose={handleCloseProject}
          onClick={handleClickUser}
          projectname={projectName}
        />
      )}
      {userModalVisible && (
        <MemberInfoModal
          onClose={handleCloseUser}
          username={username}
          projectName={projectName}
          role={userRole}
        />
      )}
      <div>
        <InputField
          placeholder={"Search"}
          type={"Text"}
          value={search}
          onChange={(e) => {
            setSearch(e.target.value);
          }}
        />
        <ul className="mt-3 border-2 text-left border-black rounded-2xl px-2 divide-y divide-gray-300 font-semibold text-[30px] cursor-pointer overflow-auto h-[60vh] w-[40vw]">
          {props.projects
            .filter((project) => {
              return search.toLowerCase() === ""
                ? project.name
                : project.name.toLowerCase().includes(search.toLowerCase());
            })
            .map((project) => (
              <li
                className="hover:font-extrabold hover:underline p-1"
                key={project.name}
                onClick={() => {
                  handleClickProject(project.name);
                }}
              >
                {project.name}
              </li>
            ))}
        </ul>
      </div>
    </>
  );
}
