import { useState } from "react";
import UserInfoModal from "./UserInfoModal";
import InputField from "./InputField";

/**
 * A list of users for admin manage users page, that sets an onClick
 * function for eact user <li> element, which displays a modul with
 * user info.
 * @param props - An array of users users to display
 * @returns {JSX.Element} The user list
 * @example
 * const users = [{ id: 1, userName: "ExampleName" }];
 * return <UserList users={users} />;
 */

export function UserListAdmin(props: { users: string[] }): JSX.Element {
  const [modalVisible, setModalVisible] = useState(false);
  const [username, setUsername] = useState("");
  const [search, setSearch] = useState("");

  const handleClick = (username: string): void => {
    setUsername(username);
    setModalVisible(true);
  };

  const handleClose = (): void => {
    setUsername("");
    setModalVisible(false);
  };

  return (
    <>
      <h1 className="font-bold text-[30px] mb-[20px]">Manage Users</h1>
      <UserInfoModal
        onClose={handleClose}
        isVisible={modalVisible}
        username={username}
      />
      <div>
        <InputField
          placeholder={"Search"}
          type={"Text"}
          value={search}
          onChange={(e) => {
            setSearch(e.target.value);
          }}
        />
        <ul className="mt-3 border-2 text-left border-black rounded-2xl px-2 divide-y divide-gray-300 font-semibold text-[30px] transition-all cursor-pointer overflow-auto h-[60vh] w-[40vw]">
          {props.users
            .filter((user) => {
              return search.toLowerCase() === ""
                ? user
                : user.toLowerCase().includes(search.toLowerCase());
            })
            .map((user) => (
              <li
                className="hover:font-extrabold hover:underline p-1"
                key={user}
                onClick={() => {
                  handleClick(user);
                }}
              >
                {user}
              </li>
            ))}
        </ul>
      </div>
    </>
  );
}
